//********************************************************************
//*** ResourcePool/ResourcePoolStats.hh
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: ResourcePoolStats.hh,v 1.3 2008/01/27 16:42:28 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_RESOURCEPOOLSTATS_HH
#define RESOURCEPOOL_RESOURCEPOOLSTATS_HH

#include <ostream>

namespace fatalmind {

class ResourcePoolStats {
	public:
		ResourcePoolStats();
		ResourcePoolStats(const ResourcePoolStats& rho, int usedSize, int freeSize);
		~ResourcePoolStats();

		unsigned long int getCalls() const;
		unsigned long int waitEvents() const;
		unsigned long int failCalls() const;
		int usedSize() const;
		int freeSize() const;
		float waitRatio() const;
		float failRatio() const;

		/**
		 * This operator is intended to build the difference of two
		 * Stats taken at two points in time.
		 * <code>
		 * ResourcePoolStats stats1 = pool.getStats();
		 * // stats include statistics for the lifetime of the pool 
		 * // [ some code working with pool ]
		 * ResourcePoolStats stats2 = pool.getStats();
		 * // new stats
		 * stats2 -= stats1;
		 * // stats2 new includes the stats between the first and the
		 * // second call to getStats();
		 * </code>
		 * Please keep in mind that the usedSize and freeSize values
		 * do not have a counter semantic. Therefore this operator does
		 * not change those values at all. In the above example this makes
		 * sense in that way that the difference value represents the more
		 * recent usedSize and freeSize values.
		 */
		ResourcePoolStats& operator-=(const ResourcePoolStats&);
		/**
		 * This operator is intended to build the sum of multiple pool
		 * stats. The result is a sum of each individual field.
		 */
		ResourcePoolStats& operator+=(const ResourcePoolStats&);

		// those methods are public so that ResourcePool<XX> can access
		// but undocumented since it is not inteded for everybody's use.
		void getCall_(); 
		void waitEvent_();
		void failCall_();

	private:
		unsigned long int getCalls_;
		unsigned long int waitEvents_;
		unsigned long int failCalls_;
		int usedSize_;
		int freeSize_;
};


} // namespace fatalmind

std::ostream& operator<<(std::ostream& os, const fatalmind::ResourcePoolStats& stats);

#endif
