//********************************************************************
//*** ResourcePool/oracle/SQLSelect.cc
//*** Copyright (c) 2003-2008 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.cc,v 1.30 2008/01/27 16:42:29 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_SQLSELECT_HH
#include "SQLSelect.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEREFTYPE_HH
#include "oracleRefType.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_SELECTLISTMETADATA_HH
#include "SelectListMetaData.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_COMMANDINTERFACE_CC
#include "CommandInterface.cc"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCEPOOL_HH
#include "ResourcePool.hh"
#endif

namespace fatalmind {
namespace oracle {

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const std::string& SQL) 
: SQLSelectRow<TM, SLMDC>(SQL)
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const ResourcePool<ResourceType<TM> >& p, const std::string& SQL) 
: SQLSelectRow<TM, SLMDC>(SQL)
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::SQLSelect(const SQLSelect& rho) 
: SQLSelectRow<TM, SLMDC>(rho) 
{
}

template<class TM, template<class> class SLMDC>
SQLSelect<TM, SLMDC>::~SQLSelect()
{
}

template<class TM, template<class> class SLMDC>
void SQLSelect<TM, SLMDC>::_execute(oracleResource<TM>& rhand, StatementCache& sthcache) 
{
	oracleHandlesWithCache<TM>& hand = rhand.get_plain_resource();
	sword ret;
	prepare_buffer(hand, sthcache);

	real_bind(hand);
	preSelectBufferBind(hand);

	/* execute statement */
	if ((ret = OCIStmtExecute(hand.getSvcCtx()
		, _sth->get()
		, hand.getError()
		, (ub4) 0
		, (ub4) 0
		, (CONST OCISnapshot *) 0
		, (OCISnapshot *) 0
		, (ub4) ((hand.getTxDepth() == 0) ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT))
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIStmtExecute", hand, ret, _SQL);
	}

	internal::registerFinish fin(hand, *this, *_sth, _SQL);
	postSelectBuffer(hand);
	while (fetch(hand, *_sth, _SQL) && process()) {
	}
}

template class SQLSelect<DefaultThreadedModel, oracle::SelectListMetaDataCache >;
template class SQLSelect<DefaultThreadedModel, oracle::SelectListMetaDataNopCache >;

} // namespace oracle
} // namespace fatalmind
