#include "t/main.hh"
#include "Cache.hh"
#include "Cache.cc" // for instantiations

#include <string>
#include <sstream>
#include <iostream>

class TestExpirationPolicy {
    public:
        TestExpirationPolicy()
		: _eff(++_no)
        {
        };

        bool isValid() const throw () {
            return true;
        };

        int efficiency() const throw() {
            return _eff;
        }

        void hit() throw() {
        };

    private:
        static unsigned int _no;
		unsigned int _eff;
};

unsigned int TestExpirationPolicy::_no;

class Ctor {
	public:
		Ctor(const char*const str) 
		: _str(str)
		{
		}

		std::string* operator()() const {
			return new std::string(_str);
		}
	private:
		const char* const _str;
};


typedef fatalmind::Cache<std::string, std::string, Ctor, TestExpirationPolicy> mycache_t;

mycache_t c10    (10);
mycache_t c100   (108);
mycache_t c1000  (1080);
mycache_t c10000 (10800);
mycache_t c100000(108000);
const std::string keypref("ksdfksdjiojsifojsadf ");

void fillup(mycache_t& c, int no) {
	for (int i = 0; i < no; ++i) {
		ostringstream os;
		os << keypref<< i;
		c.get(os.str(), "he?");		
	}
}

void INIT_Cache() {
	fillup(c10, 10);
	fillup(c100, 100);
	fillup(c1000, 1000);
	fillup(c10000, 10000);
	fillup(c100000, 100000);
}

void X_TEST_Cache_reinit() {
	INIT_Cache();
}

void TEST_Cache_one() {
	mycache_t c;
	mycache_t::value_t val;	
	val = c.get("hirsch");
	assertTrue(val.isNull());
	val = c.get("hirsch", Ctor("reh"));
	assertFalse(val.isNull());
	assertEqual(*val, "reh");
	val = c.get("hirsch", "gnu");
	assertFalse(val.isNull());
	assertEqual(*val, "reh");
	val = c.get("gnu", "hirsch");
	assertFalse(val.isNull());
	assertEqual(*val, "hirsch");
	val = c.get("hirsch");
	assertFalse(val.isNull());
	assertEqual(*val, "reh");
	c.put("hirsch", mycache_t::value_t(new std::string("neuer hirsch")));
	val = c.get("hirsch");
	assertFalse(val.isNull());
	assertEqual(*val, "neuer hirsch");
}

void TEST_Cache_invalidate() {
	mycache_t c;
	mycache_t::value_t val;	

	val = c.get("hirsch");
	assertTrue(val.isNull());

	val = c.get("hirsch", Ctor("reh"));
	assertFalse(val.isNull());
	assertEqual(*val, "reh");

	c.invalidate("hirsch");
	val = c.get("hirsch");
	assertTrue(val.isNull());

	val = c.get("hirsch", Ctor("reh"));
	assertFalse(val.isNull());
	assertEqual(*val, "reh");

	val = c.get("hirsch", "gnu");
	assertFalse(val.isNull());
	assertEqual(*val, "reh");
}

void TEST_Cache_limit() {
	mycache_t c(5);
	mycache_t::value_t val;	
	c.get("hirsch 1", "reh 1");
	c.get("hirsch 2", "reh 2");
	c.get("hirsch 3", "reh 3");
	c.get("hirsch 4", "reh 4");
	c.get("hirsch 5", "reh 5");
	
	val = c.get("hirsch 1");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 1");

	val = c.get("hirsch 2");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 2");
	
	val = c.get("hirsch 3");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 3");
	
	val = c.get("hirsch 4");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 4");
	
	val = c.get("hirsch 5");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 5");

	val = c.get("hirsch 6", "reh 6");
	assertFalse(val.isNull());
	assertEqual(*val, "reh 6");

	assertTrue(c.get("hirsch 1").isNull());
	assertFalse(c.get("hirsch 2").isNull());
	assertFalse(c.get("hirsch 3").isNull());
	assertFalse(c.get("hirsch 4").isNull());
	assertFalse(c.get("hirsch 5").isNull());
}

void int_Cache_hit_perf_no(mycache_t& c, int no) {
	mycache_t::value_t val;	
	for (int i = 0; i < 50000; ++i) {
		int j=(int) ((no*1.0)*rand()/(RAND_MAX+1.0));	
		ostringstream os;
		os << keypref << j;
		val = c.get(os.str());
		assertFalse(val.isNull());
	}
}

void int_Cache_miss_perf_no(mycache_t& c, int no) {
	mycache_t::value_t val;	
	for (int i = 0; i < 50000; ++i) {
		int j=1+2*no+(int) ((no*1.0)*rand()/(RAND_MAX+1.0));	
		ostringstream os;
		os << keypref << j;
		val = c.get(os.str());
		assertTrue(val.isNull());
	}
}

void int_Cache_update_perf_no(mycache_t& c, int no) {
	mycache_t::value_t val;	
	for (int i = 0; i < 50000; ++i) {
		int j=(int) ((no*1.0)*rand()/(RAND_MAX+1.0));	
		ostringstream os;
		os << keypref << j;
		val = c.get(os.str(), "holladrioa");
		assertFalse(val.isNull());
	}
	fillup(c, no);
}

void TEST_PERF_Cache_hit_10() {
	int_Cache_hit_perf_no(c10, 10);
}

void TEST_PERF_Cache_hit_100() {
	int_Cache_hit_perf_no(c100, 100);
}

void TEST_PERF_Cache_hit_1000() {
	int_Cache_hit_perf_no(c1000, 1000);
}

void TEST_PERF_Cache_hit_10000() {
	int_Cache_hit_perf_no(c10000, 10000);
}

void TEST_PERF_MULTI_Cache_hit_100000() {
	int_Cache_hit_perf_no(c100000, 100000);
}


void TEST_PERF_Cache_miss_10() {
	int_Cache_miss_perf_no(c10, 10);
}

void TEST_PERF_Cache_miss_100() {
	int_Cache_miss_perf_no(c100, 100);
}

void TEST_PERF_Cache_miss_1000() {
	int_Cache_miss_perf_no(c1000, 1000);
}

void TEST_PERF_Cache_miss_10000() {
	int_Cache_miss_perf_no(c10000, 10000);
}

void TEST_PERF_MULTI_Cache_miss_100000() {
	int_Cache_miss_perf_no(c100000, 100000);
}

void TEST_PERF_Cache_update_10() {
	int_Cache_update_perf_no(c10, 10);
}

void TEST_PERF_Cache_update_100() {
	int_Cache_update_perf_no(c100, 100);
}

void TEST_PERF_Cache_update_1000() {
	int_Cache_update_perf_no(c1000, 1000);
}

void TEST_PERF_Cache_update_10000() {
	int_Cache_update_perf_no(c10000, 10000);
}

void TEST_PERF_MULTI_Cache_update_100000() {
	int_Cache_update_perf_no(c100000, 100000);
}


